/**
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.PopupListener;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.RootPanel;

/**
 * @author jsugg
 *
 */
public abstract class AbstractRepositoryPopup extends DialogBox implements PopupListener {

    protected Widget parent = null;
    
    private List<ICloseListener> closeListeners = new ArrayList<ICloseListener>();

    private boolean cancelled = false;
    private static final String STYLE_NAME = "abstractPopup";
    private boolean isDragging = false;
    private FadeAnimation fadeAnimation;
    private FocusPanel dummyFocusPanel;
    
    //a container for the sub-class-specified content
    private FlowPanel content;
    private FlowPanel buttonPanel;
    
    // indicates if popup will have a "Close" button
    private boolean hasCloser = true;

    // Preload the corners to popup window.
    public static void preloadImages() {
        AbstractRepositoryPopup dialogBox = new AbstractRepositoryPopup(){

            {
                Closer closer = new Closer();
                RootPanel.getBodyElement().appendChild(closer.getElement());
                closer.getElement().getStyle().setProperty("position", "absolute");
                closer.getElement().getStyle().setProperty("left", "-9999px");
            }

            protected List<LibraryButton> getButtons() {
                return null;
            }

            protected Widget getContents() {
                return null;
            }

            protected String getTitleText() {
                return null;
            }
        };

        RootPanel.getBodyElement().appendChild(dialogBox.getElement());
        dialogBox.getElement().getStyle().setProperty("position", "absolute");
        dialogBox.getElement().getStyle().setProperty("left", "-9999px");

        LibraryButton libraryButton = new LibraryButton();
        RootPanel.getBodyElement().appendChild(libraryButton.getElement());
        libraryButton.getElement().getStyle().setProperty("position", "absolute");
        libraryButton.getElement().getStyle().setProperty("left", "-9999px");
    }



    /**
     * Which field to focus on when the popup loads up
     */
    protected void focusOnDefault() {
    }

    /**
     * What action to do if the user pushes the enter button
     * Sub classes should over ride this with the correct functionality
     */
    protected void onEnterKey() {
        dummyFocusPanel.setFocus(true);
    }

    /**
     * Allow the sub-class to specify the buttons for this popup.
     * @return   a list of GWT Button objects
     */
    protected abstract List<LibraryButton> getButtons();

    /**
     * Allow the sub-class to specify the contents of the popup.
     * @return   a widget containing the guts of the popup
     */
    protected abstract Widget getContents();

    /**
     * Allow the sub-class to specify the title to display for this popup
     * @return   a String title
     */
    protected abstract String getTitleText();

    public AbstractRepositoryPopup() {
        super(false, true);
        this.setAnimationEnabled(true);
        fadeAnimation = new FadeAnimation(this.getElement());
    }
    
    public AbstractRepositoryPopup(boolean hasCloser) {
        this();
        this.hasCloser = hasCloser; 
    }

    public AbstractRepositoryPopup(Widget parent) {
        this();
        this.parent = parent;
    }
    
    protected String getContainerStyle() {
        return "white_wrapper";
    }

    protected String getContentStyle() {
        return "white_content_deploy";
    }
    
    protected String getInnerContentStyle() {
        return "inner_content";
    }
    
    protected void rebuildPanel() {
        this.clear();

        //TODO: Add fader back in at some point
        //Repository.getRepositoryPanel().fade();

        //this.setStyleName(getContainerStyle());
        DOM.setStyleAttribute(getElement(), "display", "block");

        FlowPanel container = new FlowPanel();
        container.setStyleName(STYLE_NAME);
        setWidget(container);

        // attach the close button in the upper right corner
        if (hasCloser) {
            Closer closer = new Closer();
            container.add(closer);
        }

        // build a container for the sub-class-specified content
        content = new FlowPanel();
        container.add(content);
        content.setStyleName(getContentStyle());

        // See THU-2831
        dummyFocusPanel = new FocusPanel();
        dummyFocusPanel.setStyleName("dummyFocusPanel");
        content.add(dummyFocusPanel);

        this.setText(getTitleText());

        // get contents from the sub-class
        Widget contents = getContents();
        if (contents != null) {
            contents.addStyleName(getInnerContentStyle());
            content.add(contents);
        }
        
        refreshButtons();
    }
    
    protected void refreshButtons() {
        if (buttonPanel!=null)
            content.remove(buttonPanel);
        // get buttons from the sub-class
        buttonPanel = new FlowPanel();
        buttonPanel.setStyleName("abstractPopupButtonPanel");
        content.add(buttonPanel);
        
        for (LibraryButton button : getButtons()) {
            buttonPanel.add(button);
        }
        focusOnDefault();
    }

    @Override
    protected void onLoad() {
        super.onLoad();
        DOM.addEventPreview(this);
        focusOnDefault();
    }

    @Override
    protected void onUnload() {
        super.onUnload();
        DOM.removeEventPreview(this);
    }

    protected void cancel() {
        cancelled = true;
        close();
    }

    public void onPopupClosed(PopupPanel sender, boolean autoClosed) {

        if (parent == null) {
            return;
        }

        // TODO: add fader back in at some point
//        RepositoryPanel repository = Repository.getRepositoryPanel();
//        if (repository != null) {
//            repository.unFade();
//        }

        onClose();

        // notify any listeners that we're closing
        fireCloseEvent();
    }

    protected void close() {
        // We want to know when the animation has complete and closed the popup so that we can refresh the panel.
        this.addPopupListener(this);
        onClose();
        fireCloseEvent();
        this.removePopupListener(this);
        hide();
    }
    
    protected void onClose() {
    }

    public void onMouseDown(Widget sender, int x, int y) {
        super.onMouseDown(sender, x, y);
        isDragging = true;
        //this.addStyleName("disableScrollPanels");
        fadeAnimation.fadeTo(0.5, 100);
    }

    public void onMouseEnter(Widget sender) {
        super.onMouseEnter(sender);
    }

    public void onMouseLeave(Widget sender) {
        super.onMouseLeave(sender);
    }

    public void onMouseMove(Widget sender, int x, int y) {
        super.onMouseMove(sender, x, y);
    }

    public void onMouseUp(Widget sender, int x, int y) {
        super.onMouseUp(sender, x, y);
        isDragging = false;
        //this.removeStyleName("disableScrollPanels");
        fadeAnimation.fadeTo(1.0, 500);
    }

    public boolean isDragging() {
        return isDragging;
    }

    /**
     * The method shows and centers pop-up.
     */
    public void showCentered() {
        center();
        show();
    }

    protected class Closer extends FlowPanel {
        private Closer() {
            super();
            sinkEvents(Event.ONCLICK);
            setTitle(CommonUtils.getCommonMessages().getString("close"));
            setStyleName("closer");
        }

        public void onBrowserEvent(Event event) {
            super.onBrowserEvent(event);
            int type = DOM.eventGetType(event);
            if (type == Event.ONCLICK) {
                cancel();
            }
        }
    }

    public void removeCloseListener(ICloseListener listener) {
        if (closeListeners.contains(listener))
            closeListeners.remove(listener);
    }
    
    public void addCloseListener(ICloseListener listener) {
        if (!closeListeners.contains(listener))
            closeListeners.add(listener);
    }

    protected void fireCloseEvent() {
        for(ICloseListener listener : closeListeners) {
            listener.onClose(this);
        }
    }

    public interface ICloseListener {
        public void onClose(AbstractRepositoryPopup theClosed);
    }

    public boolean isCancelled() {
        return cancelled;
    }

    public boolean onEventPreview(Event event) {
        switch (DOM.eventGetType(event)) {
            case Event.ONKEYUP: {
                if (targetsToTextArea(event)) {
                    // Don't intercept keyboard events in a text area.
                } else {
                    // Process <Enter> key manually and remove it from event queue.
                    if (event.getKeyCode() == KeyboardListener.KEY_ENTER) {
                        onEnterKey();
                        return false;
                    }
                }
                break;
            }
        }
        
        return super.onEventPreview(event);
    }

    /**
     * Checks if event is targeted for text area HTML element.
     * @param event Event to check.
     * @return True if event is targeted to text area, False otherwise.
     */
    private boolean targetsToTextArea(Event event) {
        Element target = DOM.eventGetTarget(event);
        if (target.getNodeType() == Element.ELEMENT_NODE && target.getNodeName().equals("TEXTAREA")) {
            return true;
        } else {
            return false;
        }
    }
}

