/**
 * 
 */
package com.lombardisoftware.gwtcommon.client;

import static com.lombardisoftware.gwtcommon.server.CommonService.MAX_PRINCIPALS;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;
import com.lombardisoftware.gwtcommon.client.clientObjects.ComplexResult;

/**
 * @author bcampbell
 *
 */
public abstract class AddUsersAndGroupsPopup extends AbstractRepositoryPopup {

	private static final String STYLE_NAME = "addUsersAndGroupsPopup";

    protected static int USER_LIMIT = 20;
    
    /** Wait SEARCH_DELAY_MILLIS after user stopped typing before initiating search. */
    private static final int SEARCH_DELAY_MILLIS = 1200;

    protected final UsersListWidget userList = new UsersListWidget();
    
    protected final TextBox searchBox = new TextBox();
    protected String filter;

    private Label message;
    private boolean resultsLimited;
    
    /** Timer to delay search while user is still typing. */
    protected Timer searchTimer = new SearchTimer();
    private Image spinner;

    protected AsyncCallback<ComplexResult<ClientPrincipal>> getPopulateUserListCallback() {
        return populateUserListCallback;
    }
    
    private void cancelLastSearch() {
        assert !populateUserListCallback.isCancelled();
        populateUserListCallback.cancel();
        populateUserListCallback = new CancellableResultAsyncCallback();
    }

    private CancellableResultAsyncCallback populateUserListCallback = new CancellableResultAsyncCallback();
    
    protected AsyncCallback submitCallback = new AsyncCallback() {
        public void onFailure(Throwable caught) {
            ExceptionPopup.showException(caught);
        }

        public void onSuccess(Object result) {
            close();
        }
    };

    protected AsyncCallback<List<Object>> changeUserLimitCallback = new AsyncCallback<List<Object>>() {
        public void onFailure(Throwable caught) {
            ExceptionPopup.showException(caught);
        }

        public void onSuccess(List<Object> result) {
            USER_LIMIT = (Integer) result.get(1);
            if (USER_LIMIT == 0) {
                USER_LIMIT=MAX_PRINCIPALS;
            }
            loadUsersExternal((String) result.get(0));
        }
    };

    public AddUsersAndGroupsPopup(Widget parent) {
        super(parent);
    }

    protected List<LibraryButton> getButtons() {
        List<LibraryButton> buttons = new ArrayList<LibraryButton>();
        LibraryButton save = new LibraryButton(CommonUtils.getCommonMessages().getString("addSelectedButtonName"));
        CommonUtils.addAutomationId(save, "gwtcommon.client.addUsersAndGroupsPopup.libraryButton_0");
        save.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                cancelLastSearch();
                submit();
            }
        });
        buttons.add(save);
        return buttons;
    }

    protected Widget getContents() {
        FlowPanel contents = new FlowPanel();
        CommonUtils.addAutomationId(contents, "gwtcommon.client.addUsersAndGroupsPopup.flowPanel_0");
        contents.setStyleName(STYLE_NAME);
        
        FlowPanel searchBoxLabel = new FlowPanel();
        CommonUtils.addAutomationId(searchBoxLabel, "gwtcommon.client.addUsersAndGroupsPopup.flowPanel_1");
        searchBoxLabel.add(new Label(CommonUtils.getCommonMessages().getString("searchForName")));
        contents.add(searchBoxLabel);

        CommonUtils.addAutomationId(searchBox, "gwtcommon.client.addUsersAndGroupsPopup.textBox_0");
        searchBox.addStyleName("addUsersAndGroupsSearchField");
        searchBox.addKeyboardListener(new KeyboardListener() {
            public void onKeyDown(Widget sender, char keyCode, int modifiers) {
            }

            public void onKeyPress(Widget sender, char keyCode, int modifiers) {
            }

            public void onKeyUp(Widget sender, char keyCode, int modifiers) {
            	// Don't react on cursor movement keys
            	if (keyCode == KeyboardListener.KEY_LEFT ||
            			keyCode == KeyboardListener.KEY_RIGHT ||
            			keyCode == KeyboardListener.KEY_UP ||
            			keyCode == KeyboardListener.KEY_DOWN ||
            			keyCode == KeyboardListener.KEY_HOME ||
            			keyCode == KeyboardListener.KEY_END) {
            		return;
            	}
            	// Prevent re-loading of users if filter is not changed (f.x. when filter text box gains focus).
            	String filter = searchBox.getText();
            	if (getFilter() == filter) {
            		return;
            	}
                setFilter(filter);
            }
        });
        
        Grid searchGrid = new Grid(1, 3);
        
        searchGrid.setWidget(0, 0, searchBox);
        searchGrid.setStyleName("searchGrid");

        Label searchInstructions = new Label();
        searchInstructions.setText(CommonUtils.getCommonMessages().getString("searchBoxInstructions"));
        CommonUtils.addAutomationId(searchInstructions, "gwtcommon.client.addUsersAndGroupsPopup.label_0");
        searchInstructions.setStyleName("searchInstructions");
        
        searchGrid.setWidget(0, 1, searchInstructions);
        
        spinner = new Image("images/spinner.gif");
        spinner.getElement().getStyle().setProperty("float", "right");
        spinner.setStyleName("principalSearchSpinner");
        spinner.setVisible(false);
        searchGrid.setWidget(0, 2, spinner);
        
        contents.add(searchGrid);
        
        ClearFloatsDiv floatsDiv = new ClearFloatsDiv();
        floatsDiv.setHeight("10px");
        contents.add(floatsDiv);
        
        FlowPanel resultsBoxLabel = new FlowPanel();
        CommonUtils.addAutomationId(resultsBoxLabel, "gwtcommon.client.addUsersAndGroupsPopup.flowPanel_2");
        Label resultsLabel = new Label(CommonUtils.getCommonMessages().getString("results"));
        resultsLabel.addStyleName("inline");
        resultsBoxLabel.add(resultsLabel);
        contents.add(resultsBoxLabel);

        message = new Label("");
        message.addStyleName("inline");
        message.getElement().getStyle().setPropertyPx("paddingLeft", 20);
        resultsBoxLabel.add(message);
        
        contents.add(userList);
        
        DeferredCommand.addCommand(new Command() {
            @Override
            public void execute() {
                searchBox.setFocus(true);
            }
        });
        return contents;
    }

    protected String getTitleText() {
        return CommonUtils.getCommonMessages().getString("addUserPopupTitle");
    }

    protected void setFilter(String filter) {
    	this.filter = filter;
        if (filter.length() >= 2) {
            // Start delay timer, or reset delay when user typed enough characters.
            searchTimer.schedule(SEARCH_DELAY_MILLIS);
        } else {
            // Cancel search if user has deleted input.
            searchTimer.cancel();
        }
    }

    @Override
    protected void close() {
        searchTimer.cancel();
        super.close();
    }

    protected String getFilter() {
    	return filter;
    }
    
    protected abstract void loadUsers(String prefix);

    protected abstract void submit();

    protected void onEnterKey() {
        super.onEnterKey();
        searchTimer.cancel();
        cancelLastSearch();
        submit();
    }

    @Override
    protected void focusOnDefault() {
        super.focusOnDefault();
        searchBox.setFocus(true);
    }

    public List<ClientPrincipal> getSelectedPrincipals() {
        return userList.getSelectedPrincipals();
    }

    protected void setResultsLimited(boolean flag, int limit) {
        if (resultsLimited == flag)
            return;
        resultsLimited = flag;
        if (resultsLimited) {
            message.setText(CommonUtils.getCommonMessages().getString("resultsLimited", limit));
            message.addStyleName("error_text");
        } else {
            message.setText(" ");
            message.removeStyleName("error_text");
        }
    }
    
    protected class UsersListWidget extends ScrollPanel {
        private List<ClientPrincipal> selectedPrincipals = new ArrayList<ClientPrincipal>();
        private UserTable container;

        public UsersListWidget() {
            super();
            this.addStyleName("resultsList");
        }

        public void refreshUsers(List<ClientPrincipal> principals) {
            this.clear();
            selectedPrincipals.clear();
            container = new UserTable(principals);
            CommonUtils.addAutomationId(container, "gwtcommon.client.addUsersAndGroupsPopup.userTable_0");
            DOM.setStyleAttribute(container.getElement(),"opacity", "0.0");
            this.add(container);
            FadeAnimation fader = new FadeAnimation(container.getElement());
            fader.fadeTo(1.0, 200);
        }

        public List<ClientPrincipal> getSelectedPrincipals() {
            return selectedPrincipals;
        }

        private class UserTable extends Grid {
            private List<ClientPrincipal> principals;

            public UserTable(List<ClientPrincipal> principals) {
                super(principals.size(), 3);
                this.principals = principals;
                if (CommonUtils.isInternetExplorer7()) {
                    this.setWidth("96%");
                }
                else {
                    this.setWidth("100%"); 
                }
                build();
            }

            private void build() {
                int i = 0;
                for(final ClientPrincipal principal : principals) {         
                    String icon = null;
                    String displayName = null;

                    switch(principal.getType()) {
                        case USER : {
                            icon = CommonUtils.COMMON_IMAGES.userIcon().getHTML();
                            displayName = principal.getFullName()!= null? principal.getFullName() + " (" + principal.getName() + ")":principal.getName();
                            break;
                        }
                        case GROUP: {
                            icon = CommonUtils.COMMON_IMAGES.groupIcon().getHTML();
                            displayName = "<b>" + principal.getName() + "</b>";
                            break;
                        }
                    }
                    HTML htmlIcon = new HTML(icon);
                    setWidget(i, 0, htmlIcon);
                    HTML htmlName = new HTML(displayName);
                    setWidget(i, 1, htmlName);

                    CheckBox select = new CheckBox();
                    select.addClickListener(new ClickListener(){
                        public void onClick(Widget sender) {
                            // Add the user to the selected list so we know what all to add when
                            // the user clicks the submit button
                            if(!selectedPrincipals.contains(principal)){
                                selectedPrincipals.add(principal);
                            }
                            else {
                                selectedPrincipals.remove(principal);
                            }
                        }
                    });
                    
                    htmlIcon.addClickListener(new ClickListener(){
                        public void onClick(Widget sender) {
                            // Add the user to the selected list so we know what all to add when
                            // the user clicks the submit button
                            if(!selectedPrincipals.contains(principal)){
                                selectedPrincipals.add(principal);
                            }
                            else {
                                selectedPrincipals.remove(principal);
                            }
                            build(); //need to rebuild to check the box
                        }
                    });

                    htmlName.addClickListener(new ClickListener(){
                        public void onClick(Widget sender) {
                            // Add the user to the selected list so we know what all to add when
                            // the user clicks the submit button
                            if(!selectedPrincipals.contains(principal)){
                                selectedPrincipals.add(principal);
                            }
                            else {
                                selectedPrincipals.remove(principal);
                            }
                            build(); //need to rebuild to check the box
                        }
                    });
                    setWidget(i, 2, select);
                    this.getCellFormatter().setHorizontalAlignment(i,2, HasHorizontalAlignment.ALIGN_RIGHT);
                    
                    if(principals.size()==1) {
                        select.setChecked(true);
                        selectedPrincipals.add(principal);
                    }
                    ++i;
                }
            }
        }
    }

    /**
     * Timer to delay search until user stops typing.
     */
    private class SearchTimer extends Timer {
        @Override
        public void run() {
            GWT.log("Search request initiated", null);
            // Cancel previous handler and set new one.
            AddUsersAndGroupsPopup.this.cancelLastSearch();
            loadUsers(filter);
        }

        @Override
        public void cancel() {
            super.cancel();
            spinner.setVisible(false);
            GWT.log("Search delay timer cancelled", null);
        }

        @Override
        public void schedule(int delayMillis) {
            super.schedule(delayMillis);
            spinner.setVisible(true);
            GWT.log("Search delay timer scheduled", null);
        }
    }

    /**
     * Callback that cancel be cancelled.
     * If cancel() is called on the class its handler will not fire.
     */
    private class CancellableResultAsyncCallback implements AsyncCallback<ComplexResult<ClientPrincipal>> {

        private boolean isCancelled = false;

        public void cancel() {
            isCancelled = true;
        }

        private boolean isCancelled() {
            return isCancelled;
        }

        public void onFailure(Throwable caught) {
            if (isCancelled) {
                GWT.log("Skipping failure of cancelled async handler", caught);
                return;
            }
            spinner.setVisible(false);
            ExceptionPopup.showException(caught);
        }

        public void onSuccess(ComplexResult<ClientPrincipal> result) {
            if (isCancelled) {
                GWT.log("Skipping cancelled async handler", null);
                return;
            }
            spinner.setVisible(false);
            GWT.log("Delivering search result with " + result.getList().size() + " principals" + (result.hasMore() ? " (and more)" : ""), null);
            setResultsLimited(result.hasMore(), result.getList().size());
            userList.refreshUsers(result.getList());
        }
    }
    
    protected void loadUsersExternal(String prefix) {
        //Do nothing. This is just to allow overriding where required.
    }
}
