package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.ClickListenerCollection;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.SourcesClickEvents;

/**
 * This is a SimplePanel that can respond to click events.  This can be used instead of a FocusPanel which causes
 * a small border to appear around it when the FocusPanel is clicked on.
 */
public class ClickPanel extends FlowPanel implements SourcesClickEvents {
    /**
     * The click listeners for this panel.
     */
    @SuppressWarnings({"MismatchedQueryAndUpdateOfCollection"})
    private ClickListenerCollection clickListeners;

    /**
     * Creates a new ClickPanel.
     */
    public ClickPanel() {
        sinkEvents(Event.ONCLICK);
    }

    /**
     * Adds the given click listener to this panel.
     */
    public void addClickListener(ClickListener listener) {
        if (clickListeners == null) {
            clickListeners = new ClickListenerCollection();
        }
        clickListeners.add(listener);
    }

    /**
     * Removes the given click listener from this panel.
     */
    public void removeClickListener(ClickListener listener) {
        if (clickListeners != null) {
            clickListeners.remove(listener);
        }
    }

    /**
     * Called when a browser event occurs.
     */
    public void onBrowserEvent(Event event) {
        switch (DOM.eventGetType(event)) {
            case Event.ONCLICK:
                if (clickListeners != null) {
                    clickListeners.fireClick(this);
                }
                break;
        }
    }
}
