/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * @author bcampbell
 * @date: Jul 11, 2008
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.animation.client.Animation;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.ui.Widget;

public class FadeAnimation extends Animation {

    /**
     * current Opacity
     */
    private double currentOpacity = -1;

    /**
     * Opacity to fade to
     */
    private double fadeTo = -1;

    /**
     * Opacity to fade from
     */
    private double fadeFrom = -1;

    /**
     * The {@link com.google.gwt.user.client.Element} being affected.
     */
    private Element curElement;

    /**
     * What to do after completion
     */
    private Command onComplete;

    public FadeAnimation(Widget w){
        this(w.getElement());
    }

    /**
     * @param curElement - the element to fade
     */
    public FadeAnimation(Element curElement) {
        this.curElement = curElement;
        this.currentOpacity = 1.0;
    }

    /**
     * @param opacity - opacity to fade to
     * @param duration - duration of the animation
     */
    public void fadeTo(double opacity, int duration) {
        fadeTo(opacity, duration, null);
    }

    /**
     * @param opacity
     * @param duration
     * @param onComplete
     */
    public void fadeTo(double opacity, int duration, Command onComplete) {
        this.fadeTo = opacity;
        this.fadeFrom = 1.0;
        this.onComplete = onComplete;

        // Get the current opacity of the element
        String domOpacity = DOM.getStyleAttribute(curElement, "opacity");
        if(domOpacity == null || domOpacity.equals("")) {
            fadeFrom = 1.0;
        }
        else {
            fadeFrom = Double.valueOf(domOpacity);
        }

        // Run the animation
        super.run(duration);
    }

    protected void onComplete() {
        super.onComplete();
        if (onComplete != null) {
            onComplete.execute();
        }
    }

    @Override
    protected void onUpdate(double progress) {
        currentOpacity = interpolate(fadeFrom-((fadeFrom-fadeTo) * progress));

        DOM.setStyleAttribute(curElement, "opacity", String.valueOf(currentOpacity));
    }

}
