package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.HasHTML;
import com.google.gwt.user.client.ui.SourcesClickEvents;
import com.google.gwt.user.client.ui.ClickListenerCollection;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.History;

/**
 * This is copy and paste of GWT Hyperlink, because it is not designed to allow for subclassing.
 * Allows the specification of the element that the anchor will be nested in. Specifying null
 * for the parent element will create a bare anchor element.
 * 
 * @deprecated If you need a text with onClick listener check {@link com.google.gwt.user.client.ui.PushButton} instead.
 * The Hyperlink should be used only for navigations that change history status.
 *
 * In other words if you're going to implement an action after which
 * pressing browser "Back" button would make sence use {@link com.google.gwt.user.client.ui.Hyperlink}
 * otherwise use {@link com.google.gwt.user.client.ui.PushButton}.
 *
 * @author tcarpenter
 * @created Jan 14, 2008, 2:57:14 PM
 */
public class FlexibleHyperlink extends Widget implements HasHTML, SourcesClickEvents {

    private Element nestedAnchor;
    private ClickListenerCollection clickListeners;
    private String targetHistoryToken;

    /**
     * Creates an empty hyperlink, with the supplied element as the parent.
     *
     * @deprecated If you need a text with onClick listener check {@link com.google.gwt.user.client.ui.PushButton} instead.
     * The Hyperlink should be used only for navigations that change history status.
     *
     * In other words if you're going to implement an action after which
     * pressing browser "Back" button would make sence use {@link com.google.gwt.user.client.ui.Hyperlink}
     * otherwise use {@link com.google.gwt.user.client.ui.PushButton}.
     */
    public FlexibleHyperlink(Element parent) {
        if (parent != null) {
            setElement(parent);
            DOM.appendChild(getElement(), nestedAnchor = DOM.createAnchor());
            CommonUtils.addAutomationId(this, "gwtcommon.client.flexibleHyperlink.element_0");
        } else {
            setElement(DOM.createAnchor());
        }
        sinkEvents(Event.ONCLICK);
        setStyleName("gwt-Hyperlink");
    }

    /**
     * Creates a hyperlink with its text and target history token specified.
     *
     * @deprecated If you need a text with onClick listener check {@link com.google.gwt.user.client.ui.PushButton} instead.
     * The Hyperlink should be used only for navigations that change history status.
     *
     * In other words if you're going to implement an action after which
     * pressing browser "Back" button would make sence use {@link com.google.gwt.user.client.ui.Hyperlink}
     * otherwise use {@link com.google.gwt.user.client.ui.PushButton}.
     *
     * @param text               the hyperlink's text
     * @param asHTML             <code>true</code> to treat the specified text as html
     * @param targetHistoryToken the history token to which it will link
     * @see #setTargetHistoryToken
     */
    public FlexibleHyperlink(Element parent, String text, boolean asHTML, String targetHistoryToken) {
        this(parent);
        if (asHTML) {
            setHTML(text);
        } else {
            setText(text);
        }
        setTargetHistoryToken(targetHistoryToken);
    }

    /**
     * Creates a hyperlink with its text and target history token specified.
     *
     * @deprecated If you need a text with onClick listener check {@link com.google.gwt.user.client.ui.PushButton} instead.
     * The Hyperlink should be used only for navigations that change history status.
     *
     * In other words if you're going to implement an action after which
     * pressing browser "Back" button would make sence use {@link com.google.gwt.user.client.ui.Hyperlink}
     * otherwise use {@link com.google.gwt.user.client.ui.PushButton}.

     * @param text               the hyperlink's text
     * @param targetHistoryToken the history token to which it will link
     */
    public FlexibleHyperlink(Element parent, String text, String targetHistoryToken) {
        this(parent);
        setText(text);
        setTargetHistoryToken(targetHistoryToken);
    }

    public void addClickListener(ClickListener listener) {
        if (clickListeners == null) {
            clickListeners = new ClickListenerCollection();
        }
        clickListeners.add(listener);
    }

    public String getHTML() {
        return DOM.getInnerHTML(getAnchorElement());
    }

    /**
     * Gets the history token referenced by this hyperlink.
     *
     * @return the target history token
     * @see #setTargetHistoryToken
     */
    public String getTargetHistoryToken() {
        return targetHistoryToken;
    }

    public String getText() {
        return DOM.getInnerText(getAnchorElement());
    }

    public void onBrowserEvent(Event event) {
        if (DOM.eventGetType(event) == Event.ONCLICK) {
            if (clickListeners != null) {
                clickListeners.fireClick(this);
            }
            History.newItem(targetHistoryToken);
            DOM.eventPreventDefault(event);
        }
    }

    public void removeClickListener(ClickListener listener) {
        if (clickListeners != null) {
            clickListeners.remove(listener);
        }
    }

    public void setHTML(String html) {
        DOM.setInnerHTML(getAnchorElement(), html);
    }

    public Element getAnchorElement() {
        return (nestedAnchor != null ? nestedAnchor : getElement());
    }

    /**
     * Sets the history token referenced by this hyperlink. This is the history
     * token that will be passed to {@link History#newItem} when this link is
     * clicked.
     *
     * @param targetHistoryToken the new target history token
     */
    public void setTargetHistoryToken(String targetHistoryToken) {
        this.targetHistoryToken = targetHistoryToken;
        DOM.setElementProperty(getAnchorElement(), "href", "#" + targetHistoryToken);
    }

    public void setText(String text) {
        DOM.setInnerText(getAnchorElement(), text);
    }
}
