package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.ButtonBase;
import com.google.gwt.user.client.Event;
import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.DivElement;
import com.google.gwt.dom.client.SpanElement;

/**
 * The class implements button with image on the left and text title.
 *
 * @author Vyacheslav Nadvorny vnadvorny
 */
public class ImageButton extends ButtonBase {
    
    private DivElement iconNode;
    private SpanElement titleSpan;
    private DivElement htmlFace;

    public ImageButton(String title) {
        this(title, null);
    }

    public ImageButton(String title, String iconClassName) {
        this(title, false, iconClassName);
    }

    public ImageButton(String title, boolean isHtml, String iconClassName) {

        super(Document.get().createDivElement());
        setStyleName("twImageButton");
        sinkEvents(Event.MOUSEEVENTS);

        htmlFace = Document.get().createDivElement();
        getElement().appendChild(htmlFace);

        iconNode = Document.get().createDivElement();
        htmlFace.appendChild(iconNode);
        if (iconClassName != null) {
            iconNode.setClassName(iconClassName + " actIcon");
        } else {
            iconNode.setClassName("actIcon");
        }

        titleSpan = Document.get().createSpanElement();
        htmlFace.appendChild(titleSpan);
        if (title != null) {
            if (isHtml) {
                titleSpan.setInnerHTML(title);
            }
            else {
                titleSpan.setInnerText(title);
            }
        } else {
            // Leave text node empty.
        }
    }

    public void setIconClass(String iconClass) {
        iconNode.setClassName(iconClass);
    }

    public void setText(String text) {
        titleSpan.setInnerText(text);
    }

    public void setHTML(String html) {
        titleSpan.setInnerHTML(html);
    }

    @Override
    public void onBrowserEvent(Event event) {
        switch (event.getTypeInt()) {
            case Event.ONMOUSEOUT:
                onMouseOut();
                break;
            case Event.ONMOUSEOVER:
                onMouseOver();
                break;
        }
        super.onBrowserEvent(event);
    }

    private void onMouseOver() {
        htmlFace.setClassName("hover");
    }

    private void onMouseOut() {
        htmlFace.setClassName("");
    }
}
