/**
 * 
 */
package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.MouseListenerAdapter;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.SourcesTabEvents;
import com.google.gwt.user.client.ui.TabListener;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.FocusListener;
import com.google.gwt.user.client.ui.PopupPanel.PositionCallback;

import com.lombardisoftware.gwtcommon.client.PipesMenu.ActionLink;

/**
 * @author jsugg
 *
 */
//public class RepositoryTabBar extends TabBar {
public class RepositoryTabBar extends FlowPanel implements SourcesTabEvents {

    protected List<Tab> tabs = new ArrayList<Tab>();
    protected List tabPanels = new ArrayList();
    protected List<TabListener> listeners = new ArrayList<TabListener>();
    private int selectedTab = 0;
    
    /**
     * 
     */
    public RepositoryTabBar(List<Tab> tabs) {
        super();
        this.tabs = tabs;
        setStyleName("tab");
        addTabs(tabs);
    }

    public void addTabs(List<Tab> tabList) {
        for (int i = 0; i < tabList.size(); i++)
            addTab(tabList.get(i));
    }
    
    public void addTab(Tab tab) {
        // add front spacer panel if
        if (tabPanels.size() == 0) {
            FlowPanel spacer = new FlowPanel();
            CommonUtils.addAutomationId(spacer, "gwtcommon.client.repositoryTabBar.flowPanel_0");
            spacer.setStyleName("tabSpacer");
            spacer.add(new HTML("&nbsp;"));
            add(spacer);
        }
        
        final FocusPanel tabPanel = new FocusPanel();
        tabPanel.addFocusListener(new FocusListener() {
            public void onFocus(Widget sender) {
                tabPanel.setFocus(false);
            }

            public void onLostFocus(Widget sender) {
            }
        });
        CommonUtils.addAutomationId(tabPanel, "gwtcommon.client.repositoryTabBar.focusPanel_tabPanel");
        tabPanels.add(tabPanel);
        final int index = tabPanels.size() - 1;
        tabPanel.setStyleName("actualtab");
        tabPanel.addStyleName("inline");
        Panel parent = tabPanel;
        
        // if we have an image, then we create a container panel to hold the image
        // and text together, since a FocucPanel only accepts a single child
        if (tab.image != null){
            parent = new FlowPanel();
            CommonUtils.addAutomationId(parent, "gwtcommon.client.repositoryTabBar.flowPanel_parent");
            parent.addStyleName("inline");
            parent.setStyleName("tabContainer");
            tabPanel.add(parent);
        }
        
        HTML html = new HTML(tab.label);
        parent.add(html);
        CommonUtils.addAutomationId(html, "gwtcommon.client.repositoryTabBar.html_" + tab.label);
        if (tab.image != null) {
//            parent.add(new Span("&nbsp;", true));            
            parent.add(tab.image);
            tab.image.setStyleName("chevronPlaced");
            tab.image.addStyleName("inline");
        }
        DOM.setElementAttribute(tabPanel.getElement(), "align", "center");        
        add(tabPanel);
        
        tabPanel.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                selectTab(index);
            }
        });
    }
    
    public void selectTab(int index) {
        // first check for vetos
        for (int i = 0; i < listeners.size(); i++) {
            TabListener listener = (TabListener) listeners.get(i);
            if (!listener.onBeforeTabSelected(this, index))
                return;
        }

        // de-select the old tab
        FocusPanel old = (FocusPanel) tabPanels.get(selectedTab);
        old.getElement().setAttribute("id", "");

        selectedTab = index;
        FocusPanel selected = (FocusPanel) tabPanels.get(selectedTab);
        selected.getElement().setAttribute("id", "tab-selected");
        // if there were no vetos then notify all listeners
        for (int i = 0; i < listeners.size(); i++) {
            TabListener listener = (TabListener) listeners.get(i);
            listener.onTabSelected(this, selectedTab);
        }
    }

    public void addTabListener(TabListener listener) {
        if (!listeners.contains(listener))
            listeners.add(listener);
    }

    public void removeTabListener(TabListener listener) {
        if (listeners.contains(listener))
            listeners.remove(listener);
    }
    
    public int getSelectedTab() {
        return selectedTab;
    }
    
    public static class Tab {
        
        public String label = "";
        public Image image = null;
        
        public Tab(String label, Image image) {
            super();
            this.label = label;
            this.image = image;
        }
    }

    public static abstract class TabPopup extends PopupPanel {
        private static final String ACTION_DIV_HTML_1 = "<div class=\"wsAction\" id=\"";
        private static final String ACTION_DIV_HTML_2 = "\"></div>\n";
            
        private boolean reallyVisible;
        private List actions = new ArrayList();
        
        /**
         * @param actions  A list of PipesMenu.ActionLink objects
         */
        public TabPopup(List /* PipesMenu.ActionLink */ actions) {
            super(true);
            // TODO: should we be dynamically retrieving this list of actions to give subclasses the opportunity to asynchronously construct them?
            this.actions = actions;
            this.setAnimationEnabled(true);
        }
        
        /**
         * Allow a subclass to specify in a potentially asynchronous fashion
         * what the extra information in the flyover should be.
         * @param callback    onSuccess should be called with a widget when construction is complete
         */
        protected abstract void getFlyoverInformation(AsyncCallback callback);
        
        protected void rebuildPopup() {
            setStyleName("wsMenuWrapper");
            FocusPanel focusContainer = new FocusPanel();
            CommonUtils.addAutomationId(focusContainer, "gwtcommon.client.repositoryTabBar.focusPanel_0");
            FlowPanel container = new FlowPanel();
            CommonUtils.addAutomationId(container, "gwtcommon.client.repositoryTabBar.flowPanel_0");
            focusContainer.add(container);
            
            // build the html to represent this mouseover
            StringBuffer html = new StringBuffer();
            // we need to add a div for each action
            for (int i =0; i < actions.size(); i++) {
                ActionLink action = (ActionLink) actions.get(i);
                html.append(ACTION_DIV_HTML_1);
                html.append(action.name);
                html.append(ACTION_DIV_HTML_2);
            }
            
            HTMLPanel htmlPanel = new HTMLPanel(html.toString());
            CommonUtils.addAutomationId(htmlPanel, "gwtcommon.client.repositoryTabBar.htmlPanel_0");
            // now add in the icon and links for each action
            for (int i = 0; i < actions.size(); i++) {
                ActionLink action = (ActionLink) actions.get(i);
                HorizontalPanel actionPanel = new HorizontalPanel();
                CommonUtils.addAutomationId(actionPanel, "gwtcommon.client.repositoryTabBar.hPanel_0");
                // build the icon, if necessary
                if (action.icon != null)
                    actionPanel.add(action.icon);
                // build the link
                Hyperlink link = new Hyperlink(action.name, "#");
                CommonUtils.addAutomationId(link, "gwtcommon.client.repositoryTabBar.link_0");
                if (action.listener != null)
                    link.addClickListener(action.listener);
                // make the link visible
                actionPanel.add(link);
                htmlPanel.add(actionPanel, action.name);
            }

            container.add(htmlPanel);
            // add the follow-on information
            final FlowPanel infoPanel = new FlowPanel();
            CommonUtils.addAutomationId(infoPanel, "gwtcommon.client.repositoryTabBar.fPanel_0");
            container.add(infoPanel);
            getFlyoverInformation(new AsyncCallback () {
                public void onFailure(Throwable caught) {
                    // TODO: log an error here
                }
                public void onSuccess(Object result) {
                    // we expect result to be a widget
                    infoPanel.add((Widget) result);
                }
            });
            
            focusContainer.addMouseListener(new MouseListenerAdapter() {
                public void onMouseLeave(Widget sender) {
                    hide();
                }
            });
            setWidget(focusContainer);
        }

        public void setPopupPositionAndShow(PositionCallback callback) {
            rebuildPopup();
            super.setPopupPositionAndShow(callback);
            reallyVisible = true;
        }

        public void hide() {
            super.hide();
            reallyVisible = false;
        }

        public void setVisible(boolean visible) {
            super.setVisible(visible);
            reallyVisible = visible;
        }

        protected void onDetach() {
            super.onDetach();
            reallyVisible = false;
        }

        //PopupPanel.isVisible LIES LIES LIES
        //GWT Issue #1665 http://code.google.com/p/google-web-toolkit/issues/detail?id=1665
        public boolean isVisible() {
            return reallyVisible;
        }

    }

    public static class MouseOverImage extends Image {

        private TabPopup popup = null;

        public MouseOverImage(String imageUrl, TabPopup popup) {
            super(imageUrl);
            this.popup = popup;
        }

        //GAH
        //GWT event handling needs some serious work.
        public void onBrowserEvent(Event event) {
            if (DOM.eventGetType(event) == Event.ONMOUSEOUT && popup != null && popup.isVisible()) {
                int mouseX = DOM.eventGetClientX(event);
                int mouseY = DOM.eventGetClientY(event);
                int popX = popup.getAbsoluteLeft();
                int popY = popup.getAbsoluteTop();

                //is it within the popup?
                if (mouseX >= popX && mouseX <= popX + popup.getOffsetWidth() && mouseY >= popY && mouseY <= popY + getOffsetHeight()) {
                    //eat the event
                    return;
                }
                popup.hide();
                
            } else if (DOM.eventGetType(event) == Event.ONMOUSEOVER && (popup == null || !popup.isVisible())) {
                popup.rebuildPopup();
                popup.setPopupPositionAndShow(new PositionCallback() {
                    public void setPosition(int offsetWidth, int offsetHeight) {
                        popup.setPopupPosition(getAbsoluteLeft() + 4, getAbsoluteTop() + 4);
                    }
                });
            } else {
                super.onBrowserEvent(event);
            }

        }
    }

}
