package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.Command;

/**
 * This command implements the GWT Command interface but wraps the execute method in a try/catch to prevent uncaught
 * exceptions from crudely hitting the browser.
 *
 * @author Austin Arnold aarnold
 */
public abstract class SafeCommand implements Command {

    /**
     * Calls doExecute and catches exceptions to be displayed by the command interface.
     */
    public void execute() {
        try {
            doExecute();
        } catch(Exception e) {
            ExceptionPopup.showException(e, "Error in SafeCommand(" + getClass().getName() + ")"); // $NON-NLS-1$
        }
    }

    /**
     * Executes the command. 
     */
    public abstract void doExecute() throws Exception;
}
