/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * @author bcampbell
 * @date: Sep 14, 2008
 */
package com.lombardisoftware.gwtcommon.client.clientObjects;

import java.util.List;
import java.util.ArrayList;

/**
 * There is very little disinction between a user and a group.  A principal represents either.
 */
public class ClientPrincipal extends AbstractClientObject {

    public enum Type {
        USER,
        GROUP
    }

    private Type type;
    private String id;
    private String name;
    private String fullName;
    private String teamManagerGroupId;
    private String teamManagerGroupName;
    private String description;
    private int numMembers;
    private List<ClientPermission> permissions = new ArrayList<ClientPermission>();
    private boolean readOnly;    
    
    public ClientPrincipal() {
    }

    public ClientPrincipal(Type type, String id, String name, String fullName, String description) {
        this.type = type;
        this.id = id;
        this.name = name;
        this.fullName = fullName;
        this.description = description;
        this.numMembers = this.type == Type.USER ? -1 : 0 ;
        this.teamManagerGroupId = null;
    }

    public ClientPrincipal(Type type, String id, String name, String fullName, String description, int numMembers, boolean readOnly) {
        this.type = type;
        this.id = id;
        this.name = name;
        this.fullName = fullName;
        this.description = description;
        this.numMembers = numMembers;
        this.readOnly = readOnly;
        this.teamManagerGroupId = null;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getFullName() {
        return fullName;
    }

    public void setFullName(String fullName) {
        this.fullName = fullName;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getUserName() {
        return name;
    }

    public String getTeamManagerGroupId() {
        return teamManagerGroupId;
    }

    public void setTeamManagerGroupId(String teamManagerGroupId) {
        this.teamManagerGroupId = teamManagerGroupId;
    }

    public String getTeamManagerGroupName() {
        if(teamManagerGroupName == null) return "";
        return teamManagerGroupName;
    }

    public void setTeamManagerGroupName(String teamManagerGroupName) {
        this.teamManagerGroupName = teamManagerGroupName;
    }

    /**
     *
     * @return -1 for if type is user, else the number of members in the group
     */
    public int getNumMembers() {
        // If it's a user, there are no members, so return -1
        if(type == Type.USER) {
            return -1;
        }
        return numMembers;
    }

    /**
     * Sets the number of members if the principle is a group.  Otherwise it's a no-op
     * @param numMembers
     */
    public void setNumMembers(int numMembers) {
        if(type != Type.USER) {
            this.numMembers = numMembers;
        }
    }

    public List<ClientPermission> getPermissions() {
        return permissions;
    }

    public void setPermissions(List<ClientPermission> permissions) {
        this.permissions = permissions;
    }

    public void addProjectPermission(ClientPermission projectPermission) {
        this.permissions.add(projectPermission);
    }

    /**
     * Convenience method to tell if the principal is a user or group.
     *
     * @return
     */
    public boolean isUser() {
        switch(type) {
            case GROUP: return false;
        }
        return true;
    }

    /**
     * Convience method to get the display name visible in user list tables
     * @return
     */
    public String getListDisplayName() {
        if(isUser()) {
            return getFullName()!= null? getFullName() + " (" + getName() + ")":getName();
        }
        else {
            return getName();
        }
    }
    
    public boolean isReadOnly() {
        return readOnly;
    }
    
    public void setReadOnly(boolean readOnly) {
        this.readOnly = readOnly;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append(isUser()? "User" : "Group");
        builder.append(": ");
        builder.append(getListDisplayName());
        builder.append(" " + getDescription() != null ? getDescription():"");
        builder.append(" teamManagerGroupId(").append(teamManagerGroupId).append(")");

        return builder.toString();
    }
}
