/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such 
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into 
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @author bcampbell
 * @date: Sep 4, 2007
 */

package com.lombardisoftware.gwtcommon.client.clientObjects;

import java.io.Serializable;

public class ClientServer extends AbstractClientObject implements Serializable, Favoritable {

    public final static int NORMAL_STATUS = 0;
    public final static int AWOL_STATUS = 1;
    public final static int UNAVAILABLE_STATUS = 2;
    
    private String id;
    private long createdByUserId;
    private String createdByUserName;
    private long createdOn;
    private long lastModifiedByUserId;
    private String lastModifiedByUserName;
    private long lastModified;
    private String name = "";
    private String address = "";
    private String port = "";
    private String description = null;
    private String providerUrl = "";
    private String designatedUser = "";
    private String designatedPassword = "";
    private Type type;
    private int instances;
    private int snapshots;
    private int deploymentsCount;
    private boolean favorite = false;
    private boolean offline = false;
    private int status = NORMAL_STATUS;
    private String processCenterInstallGroup = "";

    public enum Type {
        DEVELOPMENT,
        TEST,
        STAGING,
        PRODUCTION;
        
        public static Type getByNumber(int num) {
            for (Type type : Type.values()) {
                if (type.ordinal() == num) {
                    return type;
                }
            }
            return DEVELOPMENT;
        }
    }


    public ClientServer() {
    }

    public ClientServer(String id, long createdByUserId,
                        String createdByUserName, long createdOn, long lastModifiedByUserId,
                        String lastModifiedByUserName, long lastModified,
                        String name, String address, String description, long type,
                        String providerUrl, String port, String designatedUser, String designatedPassword, 
                        boolean offline, int deploymentsCount, String processCenterInstallGroup) {
        this.id = id;
        this.createdByUserId = createdByUserId;
        this.createdByUserName = createdByUserName;
        this.createdOn = createdOn;
        this.lastModifiedByUserId = lastModifiedByUserId;
        this.lastModifiedByUserName = lastModifiedByUserName;
        this.lastModified = lastModified;
        this.name = name;
        this.address = address;
        this.description = description;
        this.type = Type.getByNumber((int) type);
        this.providerUrl = providerUrl;
        this.port = port;
        this.designatedUser = designatedUser;
        this.designatedPassword = designatedPassword;
        this.offline = offline;
        this.deploymentsCount = deploymentsCount;
        this.processCenterInstallGroup = processCenterInstallGroup;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }


    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public String getPort() {
        return port;
    }

    public void setPort(String port) {
        this.port = port;
    }

    public long getCreatedByUserId() {
        return createdByUserId;
    }

    public void setCreatedByUserId(long createdByUserId) {
        this.createdByUserId = createdByUserId;
    }

    public long getCreatedOn() {
        return createdOn;
    }

    public String getProviderUrl() {
        return providerUrl;
    }

    public void setProviderUrl(String providerUrl) {
        this.providerUrl = providerUrl;
    }

    public void setCreatedOn(long createdOn) {
        this.createdOn = createdOn;
    }

    public long getLastModifiedByUserId() {
        return lastModifiedByUserId;
    }

    public void setLastModifiedByUserId(long lastModifiedByUserId) {
        this.lastModifiedByUserId = lastModifiedByUserId;
    }

    public long getLastModified() {
        return lastModified;
    }

    public void setLastModified(long lastModified) {
        this.lastModified = lastModified;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getDesignatedPassword() {
        return designatedPassword;
    }

    public void setDesignatedPassword(String designatedPassword) {
        this.designatedPassword = designatedPassword;
    }

    public String getDesignatedUser() {
        return designatedUser;
    }

    public void setDesignatedUser(String designatedUser) {
        this.designatedUser = designatedUser;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public int getInstances() {
        return instances;
    }

    public void setInstances(int instances) {
        this.instances = instances;
    }

    public int getSnapshots() {
        return snapshots;
    }

    public void setSnapshots(int snapshots) {
        this.snapshots = snapshots;
    }

    public String getLastModifiedByUserName() {
        return lastModifiedByUserName;
    }

    public void setLastModifiedByUserName(String lastModifiedByUserName) {
        this.lastModifiedByUserName = lastModifiedByUserName;
    }

    public String getCreatedByUserName() {
        return createdByUserName;
    }

    public void setCreatedByUserName(String createdByUserName) {
        this.createdByUserName = createdByUserName;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || !(o instanceof ClientServer)) return false;

        ClientServer that = (ClientServer) o;

        if (createdByUserId != that.createdByUserId) return false;
        if (createdOn != that.createdOn) return false;
        if (lastModified != that.lastModified) return false;
        if (lastModifiedByUserId != that.lastModifiedByUserId) return false;
        if (address != null ? !address.equals(that.address) : that.address != null) return false;
        if (description != null ? !description.equals(that.description) : that.description != null) return false;
        if (id != null ? !id.equals(that.id) : that.id != null) return false;
        if (name != null ? !name.equals(that.name) : that.name != null) return false;
        if (providerUrl != null ? !providerUrl.equals(that.providerUrl) : that.providerUrl != null) return false;
        if (designatedUser != null ? !designatedUser.equals(that.designatedUser) : that.designatedUser != null) return false;
        if (designatedPassword != null ? !designatedPassword.equals(that.designatedPassword) : that.designatedPassword != null) return false;
        if (processCenterInstallGroup != null ? !processCenterInstallGroup.equals(that.processCenterInstallGroup) : that.processCenterInstallGroup != null) return false;

        //instances intentionally excluded... client only fake data
        //snapshots intentionally excluded... client only fake data
        return true;
    }

    public int hashCode() {
        int result;
        result = (id != null ? id.hashCode() : 0);
        result = 31 * result + (int) (createdByUserId ^ (createdByUserId >>> 32));
        result = 31 * result + (int) (createdOn ^ (createdOn >>> 32));
        result = 31 * result + (int) (lastModifiedByUserId ^ (lastModifiedByUserId >>> 32));
        result = 31 * result + (int) (lastModified ^ (lastModified >>> 32));
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (address != null ? address.hashCode() : 0);
        result = 31 * result + (description != null ? description.hashCode() : 0);
        result = 31 * result + (providerUrl != null ? providerUrl.hashCode() : 0);
        result = 31 * result + (designatedUser != null ? designatedUser.hashCode() : 0);
        result = 31 * result + (designatedPassword != null ? designatedPassword.hashCode() : 0);
        result = 31 * result + (processCenterInstallGroup != null ? processCenterInstallGroup.hashCode() : 0);
        
        //instances intentionally excluded... client only fake data
        //snapshots intentionally excluded... client only fake data
        return result;
    }

    public boolean isFavorite() {
        return favorite;
    }

    public void setFavorite(boolean favorite) {
        this.favorite = favorite;
    }

    public boolean isOffline() {
        return offline;
    }

    public void setOffline(boolean offline) {
        this.offline = offline;
    }

    public int getStatus() {
        return status;
    }

    public void setStatus(int status) {
        this.status = status;
    }
    
    public int getDeploymentsCount() {
        return deploymentsCount;
    }

    public void setDeploymentsCount(int deploymentsCount) {
        this.deploymentsCount = deploymentsCount;
    }
    

    public String getProcessCenterInstallGroup() {
        return processCenterInstallGroup;
    }

    public void setProcessCenterInstallGroup(String processCenterInstallGroup) {
        this.processCenterInstallGroup = processCenterInstallGroup;
    }

    public ClientServer cloneServer() {        
        return new ClientServer(this.id,this.createdByUserId,this.createdByUserName,this.createdOn,this.lastModifiedByUserId,this.lastModifiedByUserName,this.lastModified,
                this.name,this.address,this.description,this.type.ordinal(),this.providerUrl,this.port,this.designatedUser,this.designatedPassword,this.offline,this.deploymentsCount, this.processCenterInstallGroup);
    }
}
