package com.lombardisoftware.gwtcommon.client.clientObjects;

import java.io.Serializable;

/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 * <p/>
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 * <p/>
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @author bcampbell
 * @date: Feb 6, 2008
 */

public class Sorter implements Serializable {

    private static final long serialVersionUID = -6204594509156865656L;
    
    // Current sorts that need to be supported are 

    // "Recently Opened", "Recently Updated", "Name" for "Process Apps" and "Toolkits".
    // "Date", and "Name" for Process App Snapshots.

    /** Sort Types */
    // TODO: recently opened sort is not yet implemented!
    public static final int RECENTLY_OPENED = 0;
    public static final String RECENTLY_OPENED_TAG = "RECENTLY_OPENED";
    public static final int RECENTLY_UPDATED = 1;
    public static final String RECENTLY_UPDATED_TAG = "RECENTLY_UPDATED";
    public static final int NAME = 2;
    public static final String NAME_TAG = "NAME";
    public static final int DATE = 3;
    public static final String DATE_TAG = "DATE";
    public static final int NONE = 4;
    public static final String NONE_TAG = "NONE";

    private boolean ascendingOrder = true;
    private int sortType = DATE;

    public static Sorter getNoneSort() {
        return new Sorter(NONE);
    }
    
    public Sorter(){
    }

    public Sorter(String type) {
        if (type.equals(RECENTLY_OPENED_TAG))
            sortType = RECENTLY_OPENED;
        else if (type.equals(RECENTLY_UPDATED_TAG))
            sortType = RECENTLY_UPDATED;
        else if (type.equals(NAME_TAG))
            sortType = NAME;
        else if (type.equals(DATE_TAG))
            sortType = DATE;
        else if (type.equals(NONE_TAG))
            sortType = NONE;
    }
    
    public Sorter(int sortType) {
        this.sortType = sortType;
    }
    
    public Sorter(int type, boolean ascendingOrder) {
        this.sortType = type;
        this.ascendingOrder = ascendingOrder;
    }

    public boolean isAscendingOrder() {
        return ascendingOrder;
    }

    public void setAscendingOrder(boolean ascendingOrder) {
        this.ascendingOrder = ascendingOrder;
    }

    public int getSortType() {
        return sortType;
    }

    public void setSortType(int sortType) {
        this.sortType = sortType;
    }
}
