package com.lombardisoftware.gwtcommon.client.preferences;

import java.io.Serializable;

/**
 * Stores one user preference.
 * Allows to get, set, reset to previous, store and load value.
 *
 * @author Slava Nadvorny
 */
class Preference implements Serializable {

    /** Preference value */
    private String value;

    /** Original preference value. Is different from value when changed but not saved. */
    private String originalValue;

    /** For GWT serialization */
    Preference() { }

    public Preference(String initialValue) {
        setInitialValue(initialValue);
    }

    /**
     * Returns preference value.
     * Loads value if not initialized.
     *
     * @return String preference value.
     */
    public String getValue() {
        return value;
    }

    /**
     * Sets prefs original and current value.
     *
     * @param value String preference value.
     */
    protected void setInitialValue(String value) {
        this.value = originalValue = value;
    }

    /**
     * Sets preference value but does not persist it.
     *
     * @param newValue String new preference value.
     */
    public void setValue(String newValue) {
        this.value = newValue;
    }

    /**
     * Returns true if preference has been changed but not saved.
     *
     * @return True if preference has been changed but not saved. False otherwise.
     */
    public boolean isChanged() {
        return  !originalValue.equals(value);
    }

    /**
     * Resets preference to previous value if preference has been changed but not saved.
     */
    public void reset() {
        value = originalValue;
    }
}
