package com.lombardisoftware.gwtcommon.client.preferences;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Facade for user preferences.
 * Each module using gwtcommons should define its own user references class
 * which will provide methods to load and store preference value.
 *
 * @author Slava Nadvorny
 */
public abstract class UserPreferences {

    public enum PropertyName {
        LOCALE
    }
    
    private PreferencesDataObject data;

    /**
     * @return True if at least one preference has been changed.
     */
    public boolean hasChanges() {
        for (Preference pref : getPreferences().getPrefsMap().values()) {
            if (pref.isChanged()) return true;
        }
        return false;
    }

    /**
     * Returns preference data object.
     */
    protected PreferencesDataObject getPreferences() {
        return data;
    }

    /**
     * Sets new preference value.
     * @param name Name of preference.
     * @param value New preference value.
     */
    public void setPreference(PropertyName name, String value) {
        final Preference pref = getPreferences().getPrefsMap().get(name);
        pref.setValue(value);
    }

    /**
     * Should be overridden to store preferences.
     * @param callback store handler.
     */
    abstract public void storePreferences(AsyncCallback<Void> callback);

    /**
     * Should be overridden to load user preferences.
     * @param callback load handler.
     */
    abstract public void loadPreferences(AsyncCallback<PreferencesDataObject> callback);

    public void setData(PreferencesDataObject data) {
        this.data = data;
    }

}