/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * @author bcampbell
 * @date: Oct 23, 2008
 */
package com.lombardisoftware.gwtcommon.client.security;

import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPermission;

import java.util.Set;
import java.io.Serializable;

/**
 * A security context is an object which stores the current user as well as the user's repository permissions.
 */
public class SecurityContext implements Serializable {

    private ClientPrincipal currentUser;
    private Set<ClientPermission> repoPermissions;

    public SecurityContext() {
    }

    public SecurityContext(ClientPrincipal user, Set<ClientPermission> repoPermissions) {
        this.currentUser = user;
        this.repoPermissions = repoPermissions;
    }

    public Set<ClientPermission> getRepoPermissions() {
        return repoPermissions;
    }

    public void setRepoPermissions(Set<ClientPermission> repoPermissions) {
        this.repoPermissions = repoPermissions;
    }

    public ClientPrincipal getCurrentUser() {
        return currentUser;
    }

    public void setCurrentUser(ClientPrincipal currentUser) {
        this.currentUser = currentUser;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("User: ");
        builder.append(currentUser.toString());
        builder.append("\nPermissions:" );
        for(ClientPermission p: repoPermissions) {
            builder.append(" ");
            builder.append(p.toString());
        }
        return builder.toString() ;
    }
}
