DROP PROCEDURE LSW_HOUSE_KEEPING
GO 

CREATE PROCEDURE LSW_HOUSE_KEEPING(IN modeHK DECIMAL(12,0))
    LANGUAGE SQL

MAIN:
BEGIN

DECLARE statuses VARCHAR(64);
DECLARE sqlCmd VARCHAR(4000);
DECLARE flgStatusNotExist CHAR(1);
DECLARE flgDelete CHAR(1);
DECLARE rootTaskId DECIMAL(12,0);
DECLARE taskId DECIMAL(12,0);
DECLARE taskStatus VARCHAR(2);

DECLARE at_end DECIMAL(1,0);
DECLARE not_found CONDITION for SQLSTATE '02000';
DECLARE CONTINUE HANDLER for not_found SET at_end = 1;

IF (modeHK = 1) THEN

  -- Delete orphaned and DELETED task attachments --
  DELETE FROM LSW_TASK_FILE WHERE TASK_ID IN (SELECT TASK_ID FROM LSW_TASK WHERE STATUS IN ('91','92'));
  DELETE FROM LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK_FILE) OR FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK,LSW_TASK_FILE WHERE LSW_TASK.TASK_ID = LSW_TASK_FILE.TASK_ID);
  DELETE FROM LSW_TASK_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_FILE) OR TASK_ID NOT IN (SELECT LSW_TASK.TASK_ID FROM LSW_TASK,LSW_TASK_FILE WHERE LSW_TASK.TASK_ID = LSW_TASK_FILE.TASK_ID);

  -- Delete orphaned task-scoped temporary groups --
  DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);
  DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);

ELSEIF (modeHK > 1) THEN

  -- Define what task statuses we are interested in --
  SET statuses = '';
  IF (modeHK >= 2) THEN
    SET statuses = '''91'',''92''';
  END IF;
  IF (modeHK >= 3) THEN
    SET statuses = statuses || ',''21''';
  END IF;
  IF (modeHK >= 4) THEN
    SET statuses = statuses || ',''31'',''32''';
  END IF;

  DECLARE GLOBAL TEMPORARY TABLE TEMP_DELETE_TASK (
  TASK_ID DECIMAL(12,0) NOT NULL
  ) NOT LOGGED WITH REPLACE;

  BEGIN
  -- Go through each root task id --
  DECLARE curAllRootTasks CURSOR FOR stmt1;
  SET sqlCmd = 'SELECT DISTINCT ORIG_TASK_ID FROM LSW_TASK';
  PREPARE stmt1 FROM sqlCmd;
  OPEN curAllRootTasks;
  MAIN_LOOP:
  LOOP
    SET at_end = 0;
    FETCH curAllRootTasks INTO rootTaskId;
    IF (at_end = 1) THEN
      LEAVE MAIN_LOOP;
    END IF;

    -- Check each task belonging to the current root task group. If ALL tasks in this group belong to --
    -- the statuses we are interested in, then we can remove these tasks --
    SET flgDelete = 'T';
    BEGIN
    DECLARE curRootTask CURSOR FOR stmt2;
    SET sqlCmd = 'SELECT TASK_ID, STATUS FROM LSW_TASK WHERE ORIG_TASK_ID = ?';
    PREPARE stmt2 FROM sqlCmd;
    OPEN curRootTask USING rootTaskId;
    ROOT_TASK_LOOP:
    LOOP
      SET at_end = 0;
      FETCH curRootTask INTO taskId, taskStatus;
      IF (at_end = 1) THEN
        LEAVE ROOT_TASK_LOOP;
      END IF;

      SET flgStatusNotExist = 'F';
      BEGIN
      DECLARE curStatusNotExist CURSOR FOR stmt3;
      SET sqlCmd = 'SELECT ''T'' FROM SYSIBM.SYSDUMMY1 WHERE ? NOT IN (' || statuses || ')';
      PREPARE stmt3 FROM sqlCmd;
      OPEN curStatusNotExist USING taskStatus;
      FETCH curStatusNotExist INTO flgStatusNotExist;
      CLOSE curStatusNotExist;
      END;

      IF (flgStatusNotExist = 'T') THEN
        SET flgDelete = 'F';
        LEAVE ROOT_TASK_LOOP;
      END IF;

      INSERT INTO SESSION.TEMP_DELETE_TASK VALUES (taskId);

    END LOOP;

    CLOSE curRootTask;

    IF (flgDelete = 'T') THEN
      BEGIN
      DECLARE curDeleteTask CURSOR FOR stmt4;
      SET sqlCmd = 'SELECT TASK_ID FROM SESSION.TEMP_DELETE_TASK';
      PREPARE stmt4 FROM sqlCmd;
      OPEN curDeleteTask;
      SET at_end = 0;
      DELETE_TASK_LOOP:
      LOOP
        SET at_end = 0;
        FETCH curDeleteTask INTO taskId;
        IF (at_end = 1) THEN
          LEAVE DELETE_TASK_LOOP;
        END IF;
	DELETE FROM LSW_TASK_NARR WHERE TASK_ID = taskId;
	DELETE FROM LSW_TASK_FILE WHERE TASK_ID = taskId;
	DELETE FROM LSW_TASK_ADDR WHERE TASK_ID = taskId;
	DELETE FROM LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = taskId;
	DELETE FROM LSW_TASK WHERE TASK_ID = taskId;
      END LOOP;
      CLOSE curDeleteTask;
      END;
    END IF;

    DELETE FROM SESSION.TEMP_DELETE_TASK;

    END;

  END LOOP;

  CLOSE curAllRootTasks;

  END;

  DROP TABLE SESSION.TEMP_DELETE_TASK;

  -- Delete all attachment files that are not referenced by any task --
  DELETE FROM LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK_FILE);

  -- Delete orphaned task-scoped temporary groups --
  DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);
  DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);

END IF;

END

GO

DROP PROCEDURE LSW_ERASE_TASK
GO 

CREATE PROCEDURE LSW_ERASE_TASK(IN taskId DECIMAL(12,0))
    LANGUAGE SQL

  BEGIN

    DELETE FROM LSW_TASK_ADDR WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_NARR WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_FILE WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_IPF_DATA WHERE TASK_ID = taskId;
    
    DELETE FROM LSW_TASK_EXTACT_DATA WHERE TASK_ID = taskId;
    
    DELETE FROM LSW_TASK WHERE TASK_ID = taskId;

        -- Delete orphaned task-scoped temporary groups --
    DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
      SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);
    DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
      SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);

  END

GO

