DROP PROCEDURE LSW_HOUSE_KEEPING
GO 

CREATE PROCEDURE LSW_HOUSE_KEEPING(@modeHK DECIMAL) AS

DECLARE @rootTaskId DECIMAL
DECLARE @taskId DECIMAL
DECLARE @taskStatus VARCHAR(2)
DECLARE @flgDelete CHAR(1)
DECLARE @statuses VARCHAR(64)
DECLARE @sql NVARCHAR(128)
DECLARE @sqlParams NVARCHAR(128)
DECLARE @flgStatusNotExist CHAR(1)

IF (@modeHK = 1)
  BEGIN
  /* Delete orphaned and DELETED task attachments*/
  BEGIN TRANSACTION
  DELETE FROM LSW_TASK_FILE WHERE TASK_ID IN (SELECT TASK_ID FROM LSW_TASK WHERE STATUS IN (91,92))
  DELETE FROM LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK_FILE) OR FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK,LSW_TASK_FILE WHERE LSW_TASK.TASK_ID = LSW_TASK_FILE.TASK_ID)
  DELETE FROM LSW_TASK_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_FILE) OR TASK_ID NOT IN (SELECT LSW_TASK.TASK_ID FROM LSW_TASK,LSW_TASK_FILE WHERE LSW_TASK.TASK_ID = LSW_TASK_FILE.TASK_ID)

  /* Delete orphaned task-scoped temporary groups */
  DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)
  DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)

  COMMIT TRANSACTION
  END
ELSE IF (@modeHK > 1)
  BEGIN

  /* Define what task statuses we are interested in */
  SET @statuses = ''
  IF (@modeHK >= 2) SET @statuses = '91,92'
  IF (@modeHK >= 3) SET @statuses = @statuses + ',21'
  IF (@modeHK >= 4) SET @statuses = @statuses + ',31,32'

  CREATE TABLE #TEMP_DELETE_TASK (
  TASK_ID DECIMAL(12,0) NOT NULL
  )

  /* Go through each root task id */
  DECLARE curAllRootTasks CURSOR SCROLL FOR SELECT DISTINCT ORIG_TASK_ID FROM LSW_TASK
  OPEN curAllRootTasks

  WHILE (1 = 1)
    BEGIN
    FETCH curAllRootTasks INTO @rootTaskId
    IF (@@FETCH_STATUS = -1) BREAK;

    /* Check each task belonging to the current root task group. If ALL tasks in this group belong to
        the statuses we are interested in, then we can remove these tasks */
    EXEC('DECLARE curRootTask CURSOR SCROLL FOR SELECT TASK_ID, STATUS FROM LSW_TASK WHERE ORIG_TASK_ID = ' + @rootTaskId)
    OPEN curRootTask

    SET @flgDelete = 'T'
    WHILE (1 = 1)
      BEGIN
      FETCH curRootTask INTO @taskId,@taskStatus
      IF (@@FETCH_STATUS = -1) BREAK;

      SET @flgStatusNotExist = 'F'
      SET @sql = 'SELECT @pflgStatusNotExist = ''T'' WHERE ' + @taskStatus + ' NOT IN (' + @statuses + ')'
      SET @sqlParams = '@pflgStatusNotExist CHAR OUTPUT'
      EXEC sp_executesql @sql, @sqlParams, @pflgStatusNotExist = @flgStatusNotExist OUTPUT

      IF (@flgStatusNotExist = 'T')
        BEGIN
        SET @flgDelete = 'F'
        BREAK
        END

      INSERT INTO #TEMP_DELETE_TASK VALUES (@taskId)

      END

    CLOSE curRootTask
    DEALLOCATE curRootTask

    IF (@flgDelete = 'T')
      BEGIN
      DECLARE curDeleteTask CURSOR SCROLL FOR SELECT TASK_ID FROM #TEMP_DELETE_TASK
      OPEN curDeleteTask

      WHILE (1 = 1)
        BEGIN
        FETCH curDeleteTask INTO @taskId
        IF (@@FETCH_STATUS = -1) BREAK;
        BEGIN TRANSACTION
        DELETE FROM LSW_TASK_NARR WHERE TASK_ID = @taskId
        DELETE FROM LSW_TASK_FILE WHERE TASK_ID = @taskId
        DELETE FROM LSW_TASK_ADDR WHERE TASK_ID = @taskId
        DELETE FROM LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = @taskId
        DELETE FROM LSW_TASK_IPF_DATA WHERE TASK_ID = @taskId
        DELETE FROM LSW_TASK_EXTACT_DATA WHERE TASK_ID = @taskId
        DELETE FROM LSW_TASK WHERE TASK_ID = @taskId
        COMMIT TRANSACTION
        END

      CLOSE curDeleteTask
      DEALLOCATE curDeleteTask
      END

      DELETE FROM #TEMP_DELETE_TASK

    END

  CLOSE curAllRootTasks
  DEALLOCATE curAllRootTasks

  DROP TABLE #TEMP_DELETE_TASK

  /* Delete all attachment files that are not referenced by any task */
  DELETE FROM LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK_FILE)

  /* Delete orphaned task-scoped temporary groups */
  DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)
  DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)
    
  END

GO

DROP PROCEDURE LSW_ERASE_TASK
GO 

CREATE PROCEDURE LSW_ERASE_TASK @taskId DECIMAL AS

  BEGIN

    DELETE FROM LSW_TASK_ADDR WHERE TASK_ID = @taskId

    DELETE FROM LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = @taskId

    DELETE FROM LSW_TASK_NARR WHERE TASK_ID = @taskId

    DELETE FROM LSW_TASK_FILE WHERE TASK_ID = @taskId

    DELETE FROM LSW_TASK_IPF_DATA WHERE TASK_ID = @taskId
    
    DELETE FROM LSW_TASK_EXTACT_DATA WHERE TASK_ID = @taskId
    
    DELETE FROM LSW_TASK WHERE TASK_ID = @taskId

    /* Delete task-scoped temporary group */
    DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
      SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)
    DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
      SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)

   END

GO
