CREATE OR REPLACE PROCEDURE LSW_HOUSE_KEEPING(modeHK IN NUMBER) AS

BEGIN

DECLARE

TYPE typeStatusArray IS VARRAY(5) OF VARCHAR(2);
TYPE typeDeleteId IS TABLE OF NUMBER;

rootTaskId NUMBER;
taskId NUMBER;
taskStatus VARCHAR(2);
CURSOR curRootTask (taskId IN NUMBER) IS SELECT TASK_ID, STATUS FROM LSW_TASK WHERE ORIG_TASK_ID = taskId;
CURSOR curAllRootTasks IS SELECT DISTINCT ORIG_TASK_ID FROM LSW_TASK;
i INTEGER;
statuses typeStatusArray := typeStatusArray();
deleteIds typeDeleteId := typeDeleteId();
flgDelete CHAR(1);
flgStatusExist CHAR(1);

BEGIN

IF (modeHK = 1) THEN

  /* Delete orphaned and DELETED task attachments*/
  DELETE FROM LSW_TASK_FILE WHERE TASK_ID IN (SELECT TASK_ID FROM LSW_TASK WHERE STATUS IN (91,92));
  DELETE FROM LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK_FILE) OR FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK,LSW_TASK_FILE WHERE LSW_TASK.TASK_ID = LSW_TASK_FILE.TASK_ID);
  DELETE FROM LSW_TASK_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_FILE) OR TASK_ID NOT IN (SELECT LSW_TASK.TASK_ID FROM LSW_TASK,LSW_TASK_FILE WHERE LSW_TASK.TASK_ID = LSW_TASK_FILE.TASK_ID);

  /* Delete orphaned task-scoped temporary groups */
  DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);
  DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);


ELSIF (modeHK > 1) THEN

  /* Define what task statuses we are interested in */
  IF (modeHK >= 2) THEN
    statuses.extend;
    statuses(statuses.count) := '91';
    statuses.extend;
    statuses(statuses.count) := '92';
  END IF;
  IF (modeHK >= 3) THEN
    statuses.extend;
    statuses(statuses.count) := '21';
  END IF;
  IF (modeHK >= 4) THEN
    statuses.extend;
    statuses(statuses.count) := '31';
    statuses.extend;
    statuses(statuses.count) := '32';
  END IF;

  /* Go through each root task id */
  OPEN curAllRootTasks;
  LOOP
    FETCH curAllRootTasks INTO rootTaskId;
    EXIT WHEN curAllRootTasks%NOTFOUND;

	/* Check each task belonging to the current root task group. If ALL tasks in this group belong to
	   the statuses we are interested in, then we can remove these tasks */
	flgDelete := 'T';
	deleteIds.delete;
	OPEN curRootTask (rootTaskId);
	<<lblMainLoop>>
	LOOP
      FETCH curRootTask INTO taskId,taskStatus;
      EXIT WHEN curRootTask%NOTFOUND;
	  
	  flgStatusExist := 'F';
      FOR i IN statuses.FIRST..statuses.LAST LOOP
	  IF (taskStatus = statuses(i)) THEN
	    flgStatusExist := 'T';
	    EXIT;
	  END IF;
      END LOOP;
	  IF (flgStatusExist = 'F') THEN
	    flgDelete := 'F';
	    EXIT lblMainLoop;
	  END IF;

	  /* Save the task ids to delete */
	  deleteIds.extend;
	  deleteIds(deleteIds.count) := taskId;
	END LOOP;
	CLOSE curRootTask;

	IF (flgDelete = 'T') THEN
        FOR i IN deleteIds.FIRST..deleteIds.LAST LOOP
        DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
          SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g JOIN LSW_TASK t ON (g.GROUP_ID = ABS(t.GROUP_ID) AND t.TASK_ID = deleteIds(i)) WHERE g.GROUP_TYPE = 2);
        DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
          SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g JOIN LSW_TASK t ON (g.GROUP_ID = ABS(t.GROUP_ID) AND t.TASK_ID = deleteIds(i)) WHERE g.GROUP_TYPE = 2);
	    DELETE FROM LSW_TASK_NARR WHERE TASK_ID = deleteIds(i);
	    DELETE FROM LSW_TASK_FILE WHERE TASK_ID = deleteIds(i);
	    DELETE FROM LSW_TASK_ADDR WHERE TASK_ID = deleteIds(i);
	    DELETE FROM LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = deleteIds(i);
	    DELETE FROM LSW_TASK_IPF_DATA WHERE TASK_ID = deleteIds(i);
	    DELETE FROM LSW_TASK_EXTACT_DATA WHERE TASK_ID = deleteIds(i);
	    DELETE FROM LSW_TASK WHERE TASK_ID = deleteIds(i);
	  END LOOP;
	END IF;
	
  END LOOP;

  CLOSE curAllRootTasks;

  /* Delete all attachment files that are not referenced by any task */
  DELETE FROM LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM LSW_TASK_FILE);

  /* Delete orphaned task-scoped temporary groups */
  DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);
  DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2);

END IF;

END;

END;
GO

CREATE OR REPLACE PROCEDURE LSW_BPD_INSTANCE_DELETE (bpdInstanceId IN NUMBER) AS

  taskId NUMBER;

  CURSOR tasksToDelete (bpdInstanceId IN NUMBER) IS
    SELECT TASK_ID
    FROM LSW_TASK
    WHERE BPD_INSTANCE_ID = bpdInstanceId;

  PROCEDURE eraseTask (taskId IN NUMBER) AS

  BEGIN

    DELETE FROM LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
        SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g JOIN LSW_TASK t ON (g.GROUP_ID = ABS(t.GROUP_ID) AND t.TASK_ID = taskId) WHERE g.GROUP_TYPE = 2);
    DELETE FROM LSW_USR_GRP_XREF WHERE GROUP_ID IN (
        SELECT g.GROUP_ID FROM LSW_USR_GRP_XREF g JOIN LSW_TASK t ON (g.GROUP_ID = ABS(t.GROUP_ID) AND t.TASK_ID = taskId) WHERE g.GROUP_TYPE = 2);

    DELETE FROM LSW_TASK_ADDR WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_NARR WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_FILE WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK_IPF_DATA WHERE TASK_ID = taskId;
    
    DELETE FROM LSW_TASK_EXTACT_DATA WHERE TASK_ID = taskId;

    DELETE FROM LSW_TASK WHERE TASK_ID = taskId;

  END;

  PROCEDURE eraseBpdInstance (bpdInstanceId in NUMBER) AS

  BEGIN

    DELETE FROM LSW_BPD_INSTANCE_DOC_PROPS WHERE DOC_ID IN (SELECT DOC_ID FROM LSW_BPD_INSTANCE_DOCUMENTS WHERE BPD_INSTANCE_ID = bpdInstanceId);

    DELETE FROM LSW_BPD_INSTANCE_DOCUMENTS WHERE BPD_INSTANCE_ID = bpdInstanceId;

    DELETE FROM LSW_BPD_INSTANCE_VARIABLES WHERE BPD_INSTANCE_ID = bpdInstanceId;
    
    DELETE FROM LSW_BPD_INSTANCE_DATA WHERE BPD_INSTANCE_ID = bpdInstanceId;

    DELETE FROM LSW_BPD_NOTIFICATION WHERE BPD_INSTANCE_ID = bpdInstanceId;
    
    DELETE FROM LSW_RUNTIME_ERROR WHERE BPD_INSTANCE_ID = bpdInstanceId;

    DELETE FROM LSW_BPD_INSTANCE WHERE BPD_INSTANCE_ID = bpdInstanceId;

    DELETE FROM LSW_INST_MSG_INCL WHERE BPD_INSTANCE_ID = bpdInstanceId; 

    DELETE FROM LSW_INST_MSG_EXCL WHERE BPD_INSTANCE_ID = bpdInstanceId;

  END;

BEGIN

  OPEN tasksToDelete(bpdInstanceId);

  LOOP

    FETCH tasksToDelete INTO taskId;

    EXIT WHEN tasksToDelete%NOTFOUND;

    eraseTask(taskId);

  END LOOP;

  eraseBpdInstance(bpdInstanceId);

  EXCEPTION
    WHEN OTHERS THEN
      ROLLBACK;
END;

GO
