import sys

# common operations for configuring servers

execfile(installDir+"/bin/config_util.py" )

# delete an existing server and recreate it or create a new one if needed
# all the virtual hosts called virtualHostName under the cell are also deleted
def createOrRecreateServer ( cellName, nodeName, serverName, virtualHostName, newPort, bootStrapPort ):
        global AdminConfig
        if (AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName ) != ""):
                print "There is already a server called "+serverName+" under node "+nodeName
                print "it will be removed and recreated"
                AdminConfig.remove(AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName ) )
        else:
                print "No existing server called "+serverName
                print "a new one will be created."
        #endElse 
        cell = AdminConfig.getid("/Cell:"+cellName )
        # Remove all the virtual hosts with the same name
        virtualHosts = AdminConfig.list("VirtualHost", cell ).splitlines()
        for vh in virtualHosts:
                if (AdminConfig.showAttribute(vh, "name" ) == virtualHostName):
                        AdminConfig.remove(vh )
                #endIf 
        #endFor 
        node = AdminConfig.getid("/Node:"+nodeName )
        return createServer(cell, node, serverName, virtualHostName, newPort, bootStrapPort )
#endDef 

# Create a new server with associated virtual host
# The new server is based on the default template
def createServer ( cell, node, newServerName, newVirtualHostName, newPort, bootStrapPort ):
        global AdminConfig

        serverTemplateName = "default"
        serverTemplates = AdminConfig.listTemplates("Server", "APPLICATION_SERVER" ).splitlines()

        # Find the correct template for the server
        for st in serverTemplates:
                if (AdminConfig.showAttribute(st, "name" ) == serverTemplateName):
                        serverTemplate = st
                        break 
                #endIf 
        #endFor 

        # Create the new server
        newServer = AdminConfig.createUsingTemplate("Server", node, [["name", newServerName]], serverTemplate )

        # Setup bootstrap port
        nameSpace = AdminConfig.list("NameServer", newServer )
        bootStrapAddress = AdminConfig.showAttribute(nameSpace, "BOOTSTRAP_ADDRESS" )
        bootStrapHost = AdminConfig.showAttribute(bootStrapAddress, "host" )
        AdminConfig.modify(nameSpace, [["BOOTSTRAP_ADDRESS", [["port", bootStrapPort], ["host", bootStrapHost]]]] )

        # Set Log Detail Level to stop J2CA0294W warning
        tc = AdminConfig.list("TraceService", newServer )
        AdminConfig.modify(tc, [["startupTraceSpecification", "*=info:com.ibm.ejs.j2c.ConnectionFactoryBuilderImpl=severe"]] )

        serverEntries = AdminConfig.list("ServerEntry", node ).splitlines()
        for serverEntry in serverEntries:
                sName = AdminConfig.showAttribute(serverEntry, "serverName" )
                if (sName == newServerName):
                        specialEndPoints = wsadminToList(AdminConfig.showAttribute(serverEntry, "specialEndpoints" ))
                        for specialEndPoint in specialEndPoints:
                                endPointNm = AdminConfig.showAttribute(specialEndPoint, "endPointName" )
                                if (endPointNm == "WC_defaulthost"):
                                        ePoint = AdminConfig.showAttribute(specialEndPoint, "endPoint" )
                                        AdminConfig.modify(ePoint, [["port", newPort]] )
                                        break 
                                #endIf 
                        #endFor 
                #endIf 
        #endFor 

        # Find the webcontainer for the new server
        webContainer = wsadminToList(AdminConfig.list("WebContainer", newServer ))
        webContainer = webContainer[0]

        # Create the virtual host
        newHost = AdminConfig.create("VirtualHost", cell, [["name", newVirtualHostName]] )

        # Create a host alias for the port
        AdminConfig.create("HostAlias", newHost, [["hostname", "*"], ["port", newPort]] )

        # Now link the new server and the virtual host
        AdminConfig.modify(webContainer, [["defaultVirtualHostName", newVirtualHostName]] )

        print "A server called "+newServerName+" with default virtual host "+newVirtualHostName+" has been created"

        return newServer
#endDef 

# Set the ORB to pass by reference
def setPassByReference ( server ):
        global AdminConfig

        orb = wsadminToList(AdminConfig.list("ObjectRequestBroker", server ))
        orb = orb[0]
        AdminConfig.modify(orb, [["noLocalCopies", "true"]] )
#endDef 

def getCoreGroupServerId ( coreGroup, nodeName, serverName ):
        global AdminConfig

        coreGroupServerList = wsadminToList(AdminConfig.showAttribute(coreGroup, "coreGroupServers" ))
        for coreGroupServer in coreGroupServerList:
                coreGroupServerNodeName = AdminConfig.showAttribute(coreGroupServer, "nodeName" )
                coreGroupServerServerName = AdminConfig.showAttribute(coreGroupServer, "serverName" )

                if (coreGroupServerNodeName == nodeName):
                        if (coreGroupServerServerName == serverName):
                                return coreGroupServer
                        #endIf 
                #endIf 
        #endFor
        return 0
#endDef 

# Configure logging for a server
def configureLogging(serverName, server, targetDir):
        global AdminConfig
        
        procDef = AdminConfig.list("JavaProcessDef", server)
        
        logDir = targetDir+"/logs/"+serverName
        
        # Configure redirected output logging
        stdOutFile = ["stdoutFilename", logDir+"/native_stdout.log"]
        stdErrFile = ["stderrFilename", logDir+"/native_stderr.log"]
        attrs = [stdOutFile, stdErrFile]
        AdminConfig.modify(procDef, [["ioRedirect", attrs]])
        
        # Configure system output logging
        sysOutLog = AdminConfig.showAttribute(server, "outputStreamRedirect")
        sysErrLog = AdminConfig.showAttribute(server, "errorStreamRedirect")
        AdminConfig.modify(sysOutLog, [["fileName", logDir+"/SystemOut.log"]])
        AdminConfig.modify(sysErrLog, [["fileName", logDir+"/SystemErr.log"]])

        # Configure RAS logging
        rasLoggingService = AdminConfig.list("RASLoggingService", server)
        activityLog = AdminConfig.showAttribute(rasLoggingService, "serviceLog")
        AdminConfig.modify(activityLog, [["name", logDir+"/activity.log"]])
#endDef

def deployTeamworksEar(cellName, nodeName, serverName, isCluster, clusterName, virtualHost, installDir, username, password):

        appName = "teamworks"
        
        try:
                AdminApp.uninstall(appName )
                AdminConfig.save( )
        except:
                print ""
        #endTry
                 
        if (isCluster == "false"):
                mapEJBCoreToServers = ["Teamworks Core", "twcore-ejb.jar,META-INF/ejb-jar.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                mapEJBProcessToServers = ["TeamWorks Process Beans", "twprocess-ejb.jar,META-INF/ejb-jar.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                mapEJBTaskToServers = ["TeamWorks Task Beans", "twtask-ejb.jar,META-INF/ejb-jar.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                mapTeamworksWebToServers = ["teamworks.war", "teamworks.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                mapEventMgrWebToServers = ["eventmgr.war", "eventmgr.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
        else:
                mapEJBCoreToServers = ["Teamworks Core", "twcore-ejb.jar,META-INF/ejb-jar.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                mapEJBProcessToServers = ["TeamWorks Process Beans", "twprocess-ejb.jar,META-INF/ejb-jar.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                mapEJBTaskToServers = ["TeamWorks Task Beans", "twtask-ejb.jar,META-INF/ejb-jar.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                mapTeamworksWebToServers = ["teamworks.war", "teamworks.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                mapEventMgrWebToServers = ["eventmgr.war", "eventmgr.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
        #endElse 
        modulesToServers = [mapEJBCoreToServers, mapEJBProcessToServers, mapEJBTaskToServers, mapTeamworksWebToServers, mapEventMgrWebToServers]
        
        roleMaps = [["twem", username, password], ["twuser", username, password]]
        
        if (isCluster == "false"):
                AdminApp.install(installDir+"/applications/teamworks.ear", ["-server", serverName, "-node", nodeName, "-MapRunAsRolesToUsers", roleMaps, "-defaultbinding.virtual.host", virtualHost, "-usedefaultbindings"] )
        else:
                AdminApp.install(installDir+"/applications/teamworks.ear", ["-cluster", clusterName, "-node", nodeName, "-MapRunAsRolesToUsers", roleMaps, "-defaultbinding.virtual.host", virtualHost, "-usedefaultbindings"] )
        #endElse 
        
        deployments = AdminConfig.getid("/Deployment:"+appName )
        deploymentObject = AdminConfig.showAttribute(deployments, "deployedObject" )
        classloader = AdminConfig.showAttribute(deploymentObject, "classloader" )
        
        AdminConfig.modify(classloader, [["mode", "PARENT_LAST"]] )
        AdminConfig.modify(deploymentObject, [["startingWeight", 1]])
        
        AdminConfig.save( )

#endDef

def deployPortalWar(cellName, nodeName, serverName, isCluster, clusterName, virtualHost, installDir):

        try:
                AdminApp.uninstall("portal.war" )
                AdminConfig.save( )
        except:
                print ""
        #endTry 
        # Look for the portal's WAR file in the install location and then the deploy location
        portalFile = findFile([installDir+"/applications/portal.war", installDir+"/../process-portal/portal.war"] )
        
        # Don't worry if the install fails
        try:
                # This MUST be the same as display name in web.xml
                displayName = "Teamworks Process Portal"
                webModToVH = [[displayName, "portal.war,WEB-INF/web.xml", virtualHost]]
                if (isCluster == "false"):
                        mapWebToServers = [displayName, "portal.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                else:
                        mapWebToServers = [displayName, "portal.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                #endElse 
                modulesToServers = [mapWebToServers]
                rolesToUsers = [["twuser", "No", "Yes", "", ""], ["tw_portal_admin", "No", "No", "tw_portal_admin", ""]]
                AdminApp.install(portalFile, ["-contextroot", "/portal", "-MapWebModToVH", webModToVH, "-MapModulesToServers", modulesToServers, "-MapRolesToUsers", rolesToUsers, "-appname", "portal.war", "-defaultbinding.virtual.host", virtualHost, "-usedefaultbindings"] )
                
                deployments = AdminConfig.getid("/Deployment:portal.war" )
                deploymentObject = AdminConfig.showAttribute(deployments, "deployedObject" )
                classloader = AdminConfig.showAttribute(deploymentObject, "classloader" )
                AdminConfig.modify(classloader, [["mode", "PARENT_LAST"]] )
                AdminConfig.modify(deploymentObject, [["startingWeight", 2]])
        except:
                print ""
        #endTry 

        AdminConfig.save( )
        
#endDef

def deployWebApiWar(cellName, nodeName, serverName, isCluster, clusterName, virtualHost, installDir):

        try:
                AdminApp.uninstall("webapi.war" )
                AdminConfig.save( )
        except:
                print ""
        #endTry 
        # Look for the webapi file in the install location and then the deploy location
        webapiFile = findFile([installDir+"/applications/webapi.war", installDir+"/../web-api/webapi.war"] )
        
        # Don't worry if the install fails
        try:
                # This MUST be the same as display name in web.xml
                displayName = "Lombardi Software Web API"
                webModToVH = [[displayName, "webapi.war,WEB-INF/web.xml", virtualHost]]
                if (isCluster == "false"):
                        mapWebToServers = [displayName, "webapi.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                else:
                        mapWebToServers = [displayName, "webapi.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                #endElse 
                modulesToServers = [mapWebToServers]
        
                rolesToUsers = [["twuser", "No", "Yes", "", ""]]
                AdminApp.install(webapiFile, ["-contextroot", "/webapi", "-MapWebModToVH", webModToVH, "-MapModulesToServers", modulesToServers, "-MapRolesToUsers", rolesToUsers, "-appname", "webapi.war", "-defaultbinding.virtual.host", virtualHost, "-usedefaultbindings"] )
        
                deployments = AdminConfig.getid("/Deployment:webapi.war" )
                deploymentObject = AdminConfig.showAttribute(deployments, "deployedObject" )
                classloader = AdminConfig.showAttribute(deploymentObject, "classloader" )
                AdminConfig.modify(classloader, [["mode", "PARENT_LAST"]] )
        
                webModuleList = wsadminToList(AdminConfig.showAttribute(deploymentObject, "modules" ))
                webModule = webModuleList[0]
                AdminConfig.modify(webModule, [["classloaderMode", "PARENT_LAST"]] )
                AdminConfig.modify(deploymentObject, [["startingWeight", 3]])
        except:
                print "Error: ", sys.exc_info()[0], sys.exc_info()[1]
        #endTry 

        AdminConfig.save( )

#endDef

def deployProcessAdminWar(cellName, nodeName, serverName, isCluster, clusterName, virtualHost, installDir):

        try:
                AdminApp.uninstall("ProcessAdmin.war" )
                AdminConfig.save( )
        except:
                print ""
        #endTry 
        # Look for the ProcessAdmin's WAR file in the install location and then the deploy location
        paFile = findFile([installDir+"/applications/ProcessAdmin.war", installDir+"/../process-admin/ProcessAdmin.war"] )
        
        # Don't worry if the install fails
        try:
                # This MUST be the same as display name in web.xml
                displayName = "Admin Console"
                webModToVH = [[displayName, "ProcessAdmin.war,WEB-INF/web.xml", virtualHost]]
                if (isCluster == "false"):
                        mapWebToServers = [displayName, "ProcessAdmin.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                else:
                        mapWebToServers = [displayName, "ProcessAdmin.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                #endElse 
                modulesToServers = [mapWebToServers]
                rolesToUsers = [["twuser", "No", "Yes", "", ""]]
                AdminApp.install(paFile, ["-contextroot", "/ProcessAdmin", "-MapWebModToVH", webModToVH, "-MapModulesToServers", modulesToServers, "-MapRolesToUsers", rolesToUsers, "-appname", "ProcessAdmin.war", "-defaultbinding.virtual.host", virtualHost, "-usedefaultbindings"] )
        
                deployments = AdminConfig.getid("/Deployment:ProcessAdmin.war" )
                deploymentObject = AdminConfig.showAttribute(deployments, "deployedObject" )
                classloader = AdminConfig.showAttribute(deploymentObject, "classloader" )
                AdminConfig.modify(classloader, [["mode", "PARENT_LAST"]] )
                AdminConfig.modify(deploymentObject, [["startingWeight", 4]])
        except:
                print ""
        #endTry 
        
        AdminConfig.save( )

#endDef

def deployRepositoryWar(cellName, nodeName, serverName, isCluster, clusterName, virtualHost, installDir):

        try:
                AdminApp.uninstall("repository.war" )
                AdminConfig.save( )
        except:
                print ""
        #endTry 
        # Look for the ProcessCenter's WAR file in the install location and then the deploy location
        pcFile = findFile([installDir+"/applications/repository.war", installDir+"/../process-center/repository.war"] )
        
        # Don't worry if the install fails
        try:
                # This MUST be the same as display name in web.xml
                displayName = "Repository"
                webModToVH = [[displayName, "repository.war,WEB-INF/web.xml", virtualHost]]
                if (isCluster == "false"):
                        mapWebToServers = [displayName, "repository.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",node="+nodeName+",server="+serverName]
                else:
                        mapWebToServers = [displayName, "repository.war,WEB-INF/web.xml", "WebSphere:cell="+cellName+",cluster="+clusterName]
                #endElse 
                modulesToServers = [mapWebToServers]
                rolesToUsers = [["twuser", "No", "Yes", "", ""]]
                AdminApp.install(pcFile, ["-contextroot", "/ProcessCenter", "-MapWebModToVH", webModToVH, "-MapModulesToServers", modulesToServers, "-MapRolesToUsers", rolesToUsers, "-appname", "repository.war", "-defaultbinding.virtual.host", virtualHost, "-usedefaultbindings"] )
        
                deployments = AdminConfig.getid("/Deployment:repository.war" )
                deploymentObject = AdminConfig.showAttribute(deployments, "deployedObject" )
                classloader = AdminConfig.showAttribute(deploymentObject, "classloader" )
                AdminConfig.modify(classloader, [["mode", "PARENT_LAST"]] )
                AdminConfig.modify(deploymentObject, [["startingWeight", 5]])
        except:
                print ""
        #endTry 
        
        AdminConfig.save( )

#endDef
